require 'sketchup.rb'

module LTK
  module ClearMaterial
    PLUGIN_NAME = '清除材质'
    PLUGIN_VERSION = '1.0'
    PLUGIN_AUTHOR = 'liutao'
    PLUGIN_COPYRIGHT = 'liutao91.cn'
    PLUGIN_DESCRIPTION = '将所选物体的材质恢复为默认材质。'

    # 添加元数据信息
    def self.extension_info
      {
        name: PLUGIN_NAME,
        version: PLUGIN_VERSION,
        description: PLUGIN_DESCRIPTION,
        author: PLUGIN_AUTHOR,
        copyright: PLUGIN_COPYRIGHT
      }
    end

    def self.clear_material_recursively(entity)
      if entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
        # 清除组或组件的材质
        entity.material = nil
        # 递归遍历组或组件内部的实体
        entity.definition.entities.each do |sub_entity|
          clear_material_recursively(sub_entity)
        end
      elsif entity.is_a?(Sketchup::Face)
        # 清除物体的正面材质
        entity.material = nil
        # 清除物体的背面材质
        entity.back_material = nil
      end
    end

    unless file_loaded?(__FILE__)
      # 获取当前活跃的模型
      model = Sketchup.active_model

      # 添加菜单
      UI.menu('Plugins').add_item(PLUGIN_NAME) do
        model.start_operation(PLUGIN_NAME, true) # 开始操作记录
        begin
          # 获取当前选择的实体
          selection = model.selection
          # 循环遍历选择的实体
          selection.each do |entity|
            clear_material_recursively(entity)
          end
        ensure
          model.commit_operation # 提交操作记录
        end
      end

      # 添加工具栏
      toolbar = UI::Toolbar.new(PLUGIN_NAME)
      cmd = UI::Command.new(PLUGIN_NAME) do
        model.start_operation(PLUGIN_NAME, true) # 开始操作记录
        begin
          # 获取当前选择的实体
          selection = model.selection
          # 循环遍历选择的实体
          selection.each do |entity|
            clear_material_recursively(entity)
          end
        ensure
          model.commit_operation # 提交操作记录
        end
      end
      cmd.small_icon = 'clear_material/c.png'
      cmd.large_icon = 'clear_material/c.png'
      cmd.tooltip = PLUGIN_NAME
      cmd.status_bar_text = PLUGIN_DESCRIPTION
      toolbar = toolbar.add_item(cmd)
      toolbar.show

      # 注册扩展
      extension = SketchupExtension.new(PLUGIN_NAME, 'clear_material.rb')
      extension.creator = PLUGIN_AUTHOR
      extension.version = PLUGIN_VERSION
      extension.description = PLUGIN_DESCRIPTION
      extension.copyright = PLUGIN_COPYRIGHT
      Sketchup.register_extension(extension, true)

      file_loaded(__FILE__)
    end
  end
end
